@@ -1,5 +1,4 @@
-# encoding: utf-8
-'''
+"""
 S3Manager -- Class to manage builds, corpus and queues for AFL and libFuzzer on AWS S3
 
 @author:     Christian Holler (:decoder)
@@ -11,18 +10,7 @@ License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 @contact:    choller@mozilla.com
-'''
-
-# Ensure print() compatibility with Python 3
-from __future__ import print_function
-
-from boto.s3.connection import S3Connection
-from boto.s3.key import Key
-from boto.utils import parse_ts as boto_parse_ts
-
-from tempfile import mkstemp
-from zipfile import ZipFile, ZIP_DEFLATED
-
+"""
 import hashlib
 import os
 import platform
@@ -31,11 +19,19 @@ import shutil
 import subprocess
 import sys
 import time
+from tempfile import mkstemp
+from zipfile import ZIP_DEFLATED, ZipFile
+
+from boto.s3.connection import S3Connection
+from boto.s3.key import Key
+from boto.utils import parse_ts as boto_parse_ts
 
 
-class S3Manager():
-    def __init__(self, bucket_name, project_name, build_project_name=None, zip_name="build.zip"):
-        '''
+class S3Manager:
+    def __init__(
+        self, bucket_name, project_name, build_project_name=None, zip_name="build.zip"
+    ):
+        """
         @type bucket_name: String
         @param bucket_name: Name of the S3 bucket to use
 
@@ -44,7 +40,7 @@ class S3Manager():
 
         @type cmdline_file: String
         @param cmdline_file: Path to the cmdline file to upload.
-        '''
+        """
         self.bucket_name = bucket_name
         self.project_name = project_name
         self.build_project_name = build_project_name
@@ -54,22 +50,23 @@ class S3Manager():
         self.bucket = self.connection.get_bucket(self.bucket_name)
 
         # Define some path constants that define the folder structure on S3
-        self.remote_path_queues = "%s/queues/" % self.project_name
-        self.remote_path_corpus = "%s/corpus/" % self.project_name
-        self.remote_path_corpus_bundle = "%s/corpus.zip" % self.project_name
+        self.remote_path_queues = f"{self.project_name}/queues/"
+        self.remote_path_corpus = f"{self.project_name}/corpus/"
+        self.remote_path_corpus_bundle = f"{self.project_name}/corpus.zip"
 
         if self.build_project_name:
-            self.remote_path_build = "%s/%s" % (self.build_project_name, self.zip_name)
+            self.remote_path_build = f"{self.build_project_name}/{self.zip_name}"
         else:
-            self.remote_path_build = "%s/%s" % (self.project_name, self.zip_name)
+            self.remote_path_build = f"{self.project_name}/{self.zip_name}"
 
-        # Memorize which files we have uploaded/downloaded before, so we never attempt to
-        # re-upload them to a different queue or re-download them after a local merge.
+        # Memorize which files we have uploaded/downloaded before, so we never attempt
+        # to re-upload them to a different queue or re-download them after a local
+        # merge.
         self.uploaded_files = set()
         self.downloaded_files = set()
 
     def upload_libfuzzer_queue_dir(self, base_dir, corpus_dir, original_corpus):
-        '''
+        """
         Synchronize the corpus directory of the specified libFuzzer corpus directory
         to the specified S3 bucket. This method only uploads files that don't
         exist yet on the receiving side and excludes all files in the original corpus.
@@ -81,26 +78,36 @@ class S3Manager():
         @param corpus_dir: libFuzzer corpus directory
 
         @type original_corpus: Set
-        @param original_corpus: Set of original corpus files to exclude from synchronization
-        '''
-        upload_files = [x for x in os.listdir(corpus_dir) if x not in original_corpus and x not in self.uploaded_files]
+        @param original_corpus: Set of original corpus files to exclude from
+                                synchronization
+        """
+        upload_files = [
+            x
+            for x in os.listdir(corpus_dir)
+            if x not in original_corpus and x not in self.uploaded_files
+        ]
 
         # Memorize files selected for upload
         self.uploaded_files.update(upload_files)
 
         cmdline_file = os.path.join(base_dir, "cmdline")
 
-        return self.__upload_queue_files(corpus_dir, upload_files, base_dir, cmdline_file)
+        return self.__upload_queue_files(
+            corpus_dir, upload_files, base_dir, cmdline_file
+        )
 
     def download_libfuzzer_queues(self, corpus_dir):
-        '''
-        Synchronize files from open libFuzzer queues directly back into the local corpus directory.
+        """
+        Synchronize files from open libFuzzer queues directly back into the local corpus
+        directory.
 
         @type corpus_dir: String
         @param corpus_dir: libFuzzer corpus directory
-        '''
+        """
         remote_keys = list(self.bucket.list(self.remote_path_queues))
-        remote_queues_closed_names = [x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")]
+        remote_queues_closed_names = [
+            x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")
+        ]
 
         for remote_key in remote_keys:
             # Ignore any folders
@@ -108,7 +115,9 @@ class S3Manager():
                 continue
 
             # Ignore the cmdline and closed files
-            if remote_key.name.endswith("/cmdline") or remote_key.name.endswith("/closed"):
+            if remote_key.name.endswith("/cmdline") or remote_key.name.endswith(
+                "/closed"
+            ):
                 continue
 
             (queue_name, filename) = remote_key.name.rsplit("/", 1)
@@ -128,13 +137,13 @@ class S3Manager():
                 # If the file already exists locally, ignore it
                 continue
 
-            print("Syncing from queue %s: %s" % (queue_name, filename))
+            print(f"Syncing from queue {queue_name}: {filename}")
             remote_key.get_contents_to_filename(dest_file)
 
             self.downloaded_files.add(basename)
 
     def upload_afl_queue_dir(self, base_dir, new_cov_only=True):
-        '''
+        """
         Synchronize the queue directory of the specified AFL base directory
         to the specified S3 bucket. This method only uploads files that don't
         exist yet on the receiving side.
@@ -144,7 +153,7 @@ class S3Manager():
 
         @type new_cov_only: Boolean
         @param new_cov_only: Only upload files that have new coverage
-        '''
+        """
         queue_dir = os.path.join(base_dir, "queue")
         queue_files = []
 
@@ -169,7 +178,7 @@ class S3Manager():
         return self.__upload_queue_files(queue_dir, queue_files, base_dir, cmdline_file)
 
     def download_queue_dirs(self, work_dir):
-        '''
+        """
         Downloads all queue files into the queues sub directory of the specified
         local work directory. The files are renamed to match their SHA1 hashes
         to avoid file collisions.
@@ -178,7 +187,7 @@ class S3Manager():
 
         @type work_dir: String
         @param work_dir: Local work directory
-        '''
+        """
         download_dir = os.path.join(work_dir, "queues")
 
         if not os.path.exists(download_dir):
@@ -192,9 +201,10 @@ class S3Manager():
         # Close all queues that aren't closed already.
         # This will stop the clients from uploading new data into these queues.
         #
-        # Unfortunately we have to iterate over all files in the queue path to figure out which queues exist.
-        # Then we have to determine which of them might already been closed (this shouldn't happen normally),
-        # but we should check this anyway and not consider it an error.
+        # Unfortunately we have to iterate over all files in the queue path to figure
+        # out which queues exist. Then we have to determine which of them might already
+        # been closed (this shouldn't happen normally), but we should check this anyway
+        # and not consider it an error.
         for remote_key in remote_keys:
             (queue_name, filename) = remote_key.name.rsplit("/", 1)
             remote_queue_names.add(queue_name)
@@ -204,7 +214,7 @@ class S3Manager():
         for remote_queue_name in remote_queue_names:
             if remote_queue_name not in remote_queues_already_closed:
                 closed_key = self.bucket.new_key(remote_queue_name + "/closed")
-                closed_key.set_contents_from_string('')
+                closed_key.set_contents_from_string("")
 
         for remote_key in remote_keys:
             # Ignore any folders and the closed file
@@ -213,37 +223,40 @@ class S3Manager():
 
             (queue_name, filename) = remote_key.name.rsplit("/", 1)
 
-            # This queue was closed before, assume we downloaded it before to save download requests.
+            # This queue was closed before, assume we downloaded it before to save
+            # download requests.
             if queue_name in remote_queues_already_closed:
                 continue
 
             # If we see a cmdline file, fetch it into the main work directory
-            if os.path.basename(remote_key.name) == 'cmdline':
-                remote_key.get_contents_to_filename(os.path.join(work_dir, 'cmdline'))
+            if os.path.basename(remote_key.name) == "cmdline":
+                remote_key.get_contents_to_filename(os.path.join(work_dir, "cmdline"))
                 continue
 
             tmp_file = os.path.join(download_dir, "tmp")
 
             remote_key.get_contents_to_filename(tmp_file)
 
-            with open(tmp_file, 'rb') as tmp_fd:
+            with open(tmp_file, "rb") as tmp_fd:
                 hash_name = hashlib.sha1(tmp_fd.read()).hexdigest()
 
             os.rename(tmp_file, os.path.join(download_dir, hash_name))
 
     def clean_queue_dirs(self):
-        '''
+        """
         Delete all closed remote queues.
-        '''
+        """
         remote_keys = list(self.bucket.list(self.remote_path_queues))
         remote_keys_for_deletion = []
 
-        remote_queues_closed_names = [x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")]
+        remote_queues_closed_names = [
+            x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")
+        ]
 
         for remote_key in remote_keys:
             # For folders, check if they are empty and if so, remove them
             if remote_key.name.endswith("/"):
-                #TODO: This might not work in current boto, check later
+                # TODO: This might not work in current boto, check later
                 if remote_key.size == 0:
                     remote_keys_for_deletion.append(remote_key.name)
                 continue
@@ -253,19 +266,21 @@ class S3Manager():
                 remote_keys_for_deletion.append(remote_key.name)
 
         for remote_key_for_deletion in remote_keys_for_deletion:
-            print("Deleting old key %s" % remote_key_for_deletion)
+            print(f"Deleting old key {remote_key_for_deletion}")
 
         self.bucket.delete_keys(remote_keys_for_deletion, quiet=True)
 
     def get_queue_status(self):
-        '''
+        """
         Return status data for all queues in the specified S3 bucket/project
 
         @rtype: dict
         @return: Dictionary containing queue size per queue
-        '''
+        """
         remote_keys = list(self.bucket.list(self.remote_path_queues))
-        remote_queues_closed_names = [x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")]
+        remote_queues_closed_names = [
+            x.name.rsplit("/", 1)[0] for x in remote_keys if x.name.endswith("/closed")
+        ]
 
         status_data = {}
 
@@ -275,7 +290,9 @@ class S3Manager():
                 continue
 
             # Ignore the cmdline and closed files
-            if remote_key.name.endswith("/cmdline") or remote_key.name.endswith("/closed"):
+            if remote_key.name.endswith("/cmdline") or remote_key.name.endswith(
+                "/closed"
+            ):
                 continue
 
             (queue_name, filename) = remote_key.name.rsplit("/", 1)
@@ -290,7 +307,7 @@ class S3Manager():
         return status_data
 
     def get_corpus_status(self):
-        '''
+        """
         Return status data for the corpus of the specified S3 bucket/project
 
         @type bucket_name: String
@@ -301,7 +318,7 @@ class S3Manager():
 
         @rtype: dict
         @return: Dictionary containing corpus size per date modified
-        '''
+        """
         remote_keys = list(self.bucket.list(self.remote_path_corpus))
 
         status_data = {}
@@ -322,7 +339,7 @@ class S3Manager():
         return status_data
 
     def download_build(self, build_dir):
-        '''
+        """
         Downloads build.zip from the specified S3 bucket and unpacks it
         into the specified build directory.
 
@@ -334,7 +351,7 @@ class S3Manager():
 
         @type project_name: String
         @param project_name: Name of the project folder inside the S3 bucket
-        '''
+        """
         # Clear any previous builds
         if os.path.exists(build_dir):
             shutil.rmtree(build_dir)
@@ -350,13 +367,13 @@ class S3Manager():
         subprocess.check_call(["unzip", zip_dest, "-d", build_dir])
 
     def upload_build(self, build_file):
-        '''
+        """
         Upload the given build zip file to the specified S3 bucket/project
         directory.
 
         @type build_file: String
         @param build_file: (ZIP) file containing the build that should be uploaded
-        '''
+        """
 
         if not os.path.exists(build_file) or not os.path.isfile(build_file):
             print("Error: Build must be a (zip) file.", file=sys.stderr)
@@ -364,11 +381,11 @@ class S3Manager():
 
         remote_key = Key(self.bucket)
         remote_key.name = self.remote_path_build
-        print("Uploading file %s -> %s" % (build_file, remote_key.name))
+        print(f"Uploading file {build_file} -> {remote_key.name}")
         remote_key.set_contents_from_filename(build_file)
 
     def download_corpus(self, corpus_dir, random_subset_size=None):
-        '''
+        """
         Downloads the test corpus from the specified S3 bucket and project
         into the specified directory, without overwriting any files.
 
@@ -378,7 +395,7 @@ class S3Manager():
         @type random_subset_size: int
         @param random_subset_size: If specified, only download a random subset of
                                    the corpus, with the specified size.
-        '''
+        """
         if not os.path.exists(corpus_dir):
             os.mkdir(corpus_dir)
 
@@ -396,8 +413,12 @@ class S3Manager():
 
                     with ZipFile(zip_dest, "r") as zip_file:
                         if zip_file.testzip():
-                            # Warn, but don't throw, we can try to download the corpus directly
-                            print("Bad CRC for downloaded zipfile %s" % zip_dest, file=sys.stderr)
+                            # Warn, but don't throw, we can try to download the corpus
+                            # directly
+                            print(
+                                f"Bad CRC for downloaded zipfile {zip_dest}",
+                                file=sys.stderr,
+                            )
                         else:
                             zip_file.extractall(corpus_dir)
                             return
@@ -416,7 +437,7 @@ class S3Manager():
                 remote_key.get_contents_to_filename(dest_file)
 
     def upload_corpus(self, corpus_dir, corpus_delete=False):
-        '''
+        """
         Synchronize the specified test corpus directory to the specified S3 bucket.
         This method only uploads files that don't exist yet on the receiving side.
 
@@ -425,8 +446,12 @@ class S3Manager():
 
         @type corpus_delete: bool
         @param corpus_delete: Delete all remote files that don't exist on our side
-        '''
-        test_files = [file for file in os.listdir(corpus_dir) if os.path.isfile(os.path.join(corpus_dir, file))]
+        """
+        test_files = [
+            file
+            for file in os.listdir(corpus_dir)
+            if os.path.isfile(os.path.join(corpus_dir, file))
+        ]
 
         if not test_files:
             print("Error: Corpus is empty, refusing upload.", file=sys.stderr)
@@ -434,18 +459,21 @@ class S3Manager():
 
         # Make a zip bundle and upload it
         (zip_fd, zip_dest) = mkstemp(prefix="libfuzzer-s3-corpus")
-        zip_file = ZipFile(zip_dest, 'w', ZIP_DEFLATED)
+        zip_file = ZipFile(zip_dest, "w", ZIP_DEFLATED)
         for test_file in test_files:
             zip_file.write(os.path.join(corpus_dir, test_file), arcname=test_file)
         zip_file.close()
         remote_key = Key(self.bucket)
         remote_key.name = self.remote_path_corpus_bundle
-        print("Uploading file %s -> %s" % (zip_dest, remote_key.name))
+        print(f"Uploading file {zip_dest} -> {remote_key.name}")
         remote_key.set_contents_from_filename(zip_dest)
         os.remove(zip_dest)
 
         remote_path = self.remote_path_corpus
-        remote_files = [key.name.replace(remote_path, "", 1) for key in list(self.bucket.list(remote_path))]
+        remote_files = [
+            key.name.replace(remote_path, "", 1)
+            for key in list(self.bucket.list(remote_path))
+        ]
 
         upload_list = []
         delete_list = []
@@ -462,14 +490,14 @@ class S3Manager():
         for upload_file in upload_list:
             remote_key = Key(self.bucket)
             remote_key.name = remote_path + os.path.basename(upload_file)
-            print("Uploading file %s -> %s" % (upload_file, remote_key.name))
+            print(f"Uploading file {upload_file} -> {remote_key.name}")
             remote_key.set_contents_from_filename(upload_file)
 
         if corpus_delete:
             self.bucket.delete_keys(delete_list, quiet=True)
 
     def __get_machine_id(self, base_dir, refresh=False):
-        '''
+        """
         Get (and if necessary generate) the machine id which is based on
         the current timestamp and the hostname of the machine. The
         generated ID is cached inside the base directory, so all
@@ -483,36 +511,42 @@ class S3Manager():
 
         @rtype: String
         @return: The generated/cached machine ID
-        '''
+        """
         id_file = os.path.join(base_dir, "s3_id")
 
         # We initially create a unique ID based on the hostname and the
         # current timestamp, then we store this ID in a file inside the
         # fuzzing working directory so we can retrieve it later.
         if refresh or not os.path.exists(id_file):
-            h = hashlib.new('sha1')
-            h.update(platform.node().encode('utf-8'))
-            h.update(str(time.time()).encode('utf-8'))
+            h = hashlib.new("sha1")
+            h.update(platform.node().encode("utf-8"))
+            h.update(str(time.time()).encode("utf-8"))
             id = h.hexdigest()
-            with open(id_file, 'w') as id_fd:
+            with open(id_file, "w") as id_fd:
                 id_fd.write(id)
             return id
         else:
-            with open(id_file, 'r') as id_fd:
+            with open(id_file) as id_fd:
                 return id_fd.read()
 
     def __upload_queue_files(self, queue_basedir, queue_files, base_dir, cmdline_file):
         machine_id = self.__get_machine_id(base_dir)
-        remote_path = "%s%s/" % (self.remote_path_queues, machine_id)
-        remote_files = [key.name.replace(remote_path, "", 1) for key in list(self.bucket.list(remote_path))]
+        remote_path = f"{self.remote_path_queues}{machine_id}/"
+        remote_files = [
+            key.name.replace(remote_path, "", 1)
+            for key in list(self.bucket.list(remote_path))
+        ]
 
         if "closed" in remote_files:
             # The queue we are assigned has been closed remotely.
             # Switch to a new queue instead.
-            print("Remote queue %s closed, switching to new queue..." % machine_id)
+            print(f"Remote queue {machine_id} closed, switching to new queue...")
             machine_id = self.__get_machine_id(base_dir, refresh=True)
-            remote_path = "%s%s/" % (self.remote_path_queues, machine_id)
-            remote_files = [key.name.replace(remote_path, "", 1) for key in list(self.bucket.list(remote_path))]
+            remote_path = f"{self.remote_path_queues}{machine_id}/"
+            remote_files = [
+                key.name.replace(remote_path, "", 1)
+                for key in list(self.bucket.list(remote_path))
+            ]
 
         upload_list = []
 
@@ -526,9 +560,10 @@ class S3Manager():
         for upload_file in upload_list:
             remote_key = Key(self.bucket)
             remote_key.name = remote_path + os.path.basename(upload_file)
-            print("Uploading file %s -> %s" % (upload_file, remote_key.name))
+            print(f"Uploading file {upload_file} -> {remote_key.name}")
             try:
                 remote_key.set_contents_from_filename(upload_file)
-            except IOError:
-                # Newer libFuzzer can delete files from the corpus if it finds a shorter version in the same run.
+            except OSError:
+                # Newer libFuzzer can delete files from the corpus if it finds a shorter
+                # version in the same run.
                 pass
