@@ -1,6 +1,5 @@
 #!/usr/bin/env python
-# encoding: utf-8
-'''
+"""
 choose_once -- Simulation handler that initially makes the cheapest choice and
                then keeps it for a lifetime. Region and instance type can be
                fixed with the corresponding parameters.
@@ -14,11 +13,7 @@ License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 @contact:    choller@mozilla.com
-'''
-
-# Ensure print() compatibility with Python 3
-from __future__ import print_function
-
+"""
 from .common import select_better
 
 
@@ -39,10 +34,13 @@ def run(data, sim_config, main_config):
     instance_type = ret["instance_type"]
     total_price = 0
 
-    with open("%s.log" % sim_config["name"], mode='w') as logFileFd:
+    with open(f"{sim_config['name']}.log", mode="w") as logFileFd:
         for instance_time in data[region][zone][instance_type]:
             (_, price, _) = data[region][zone][instance_type][instance_time]
             total_price = total_price + price
-            print("%s %s %s %s %s" % (region, zone, instance_type, instance_time, price), file=logFileFd)
+            print(
+                f"{region} {zone} {instance_type} {instance_time} {price}",
+                file=logFileFd,
+            )
 
     return total_price
