@@ -1,6 +1,13 @@
 from django.contrib import admin
 
-from crashmanager.models import CrashEntry, Bucket, Bug, Platform, Product, OS  # @UnresolvedImport
+from crashmanager.models import (  # @UnresolvedImport
+    OS,
+    Bucket,
+    Bug,
+    CrashEntry,
+    Platform,
+    Product,
+)
 
 admin.site.register(CrashEntry)
 admin.site.register(Bucket)
