@@ -1,28 +1,34 @@
+import json
+import logging
+import re
+from itertools import zip_longest
+
 from django.conf import settings
-from django.contrib.auth.models import User as DjangoUser, Permission
+from django.contrib.auth.models import Permission
+from django.contrib.auth.models import User as DjangoUser
 from django.contrib.contenttypes.models import ContentType
 from django.core.files.storage import FileSystemStorage
 from django.db import models
 from django.db.models.signals import post_delete, post_save
 from django.dispatch.dispatcher import receiver
 from django.utils import timezone
-import json
-import logging
-import re
-
-import six
+from enumfields import Enum, EnumField
+from notifications.signals import notify
 
 from FTB.ProgramConfiguration import ProgramConfiguration
 from FTB.Signatures.CrashInfo import CrashInfo
 from FTB.Signatures.CrashSignature import CrashSignature
 
-if getattr(settings, 'USE_CELERY', None):
+if getattr(settings, "USE_CELERY", None):
     from .tasks import triage_new_crash
 
 
 class Tool(models.Model):
     name = models.CharField(max_length=63)
 
+    def __str__(self):
+        return self.name
+
 
 class Platform(models.Model):
     name = models.CharField(max_length=63)
@@ -39,8 +45,10 @@ class OS(models.Model):
 
 
 class TestCase(models.Model):
-    test = models.FileField(storage=FileSystemStorage(location=getattr(settings, 'TEST_STORAGE', None)),
-                            upload_to="tests")
+    test = models.FileField(
+        storage=FileSystemStorage(location=getattr(settings, "TEST_STORAGE", None)),
+        upload_to="tests",
+    )
     size = models.IntegerField(default=0)
     quality = models.IntegerField(default=0)
     isBinary = models.BooleanField(default=False)
@@ -53,16 +61,16 @@ class TestCase(models.Model):
         # automatically. You must call the loadTest method if you
         # want to access the test content
 
-        super(TestCase, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def loadTest(self):
-        self.test.open(mode='rb')
+        self.test.open(mode="rb")
         self.content = self.test.read()
         self.test.close()
 
     def storeTestAndSave(self):
         self.size = len(self.content)
-        self.test.open(mode='w')
+        self.test.open(mode="w")
         self.test.write(self.content)
         self.test.close()
         self.save()
@@ -81,25 +89,47 @@ class BugProvider(models.Model):
 
     def getInstance(self):
         # Dynamically instantiate the provider as requested
-        providerModule = __import__('crashmanager.Bugtracker.%s' % self.classname, fromlist=[self.classname])
+        providerModule = __import__(
+            f"crashmanager.Bugtracker.{self.classname}", fromlist=[self.classname]
+        )
         providerClass = getattr(providerModule, self.classname)
         return providerClass(self.pk, self.hostname)
 
+    def __str__(self):
+        return self.hostname
+
 
 class Bug(models.Model):
     externalId = models.CharField(max_length=255, blank=True)
     externalType = models.ForeignKey(BugProvider, on_delete=models.deletion.CASCADE)
     closed = models.DateTimeField(blank=True, null=True)
 
+    @property
+    def tools_filter_users(self):
+        ids = User.objects.filter(
+            defaultToolsFilter__crashentry__bucket__in=self.bucket_set.all(),
+            inaccessible_bug=True,
+        ).values_list("user_id", flat=True)
+        return DjangoUser.objects.filter(id__in=ids).distinct()
+
 
 class Bucket(models.Model):
-    bug = models.ForeignKey(Bug, blank=True, null=True, on_delete=models.deletion.CASCADE)
+    bug = models.ForeignKey(
+        Bug, blank=True, null=True, on_delete=models.deletion.CASCADE
+    )
     signature = models.TextField()
     optimizedSignature = models.TextField(blank=True, null=True)
     shortDescription = models.CharField(max_length=1023, blank=True)
     frequent = models.BooleanField(blank=False, default=False)
     permanent = models.BooleanField(blank=False, default=False)
 
+    @property
+    def watchers(self):
+        ids = User.objects.filter(
+            bucketwatch__bucket=self, bucket_hit=True
+        ).values_list("user_id", flat=True)
+        return DjangoUser.objects.filter(id__in=ids).distinct()
+
     def getSignature(self):
         return CrashSignature(self.signature)
 
@@ -114,11 +144,107 @@ class Bucket(models.Model):
 
         # TODO: We could reset this only when we actually modify the signature,
         # but this would require fetching the old signature from the database again.
-        keepOptimized = kwargs.pop('keepOptimized', False)
+        keepOptimized = kwargs.pop("keepOptimized", False)
         if not keepOptimized:
             self.optimizedSignature = None
 
-        super(Bucket, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
+
+    def reassign(self, submitSave):
+        """
+        Assign all unassigned issues that match our signature to this bucket.
+        Furthermore, remove all non-matching issues from our bucket.
+
+        We only actually save if "submitSave" is set.
+        For previewing, we just count how many issues would be assigned and removed.
+        """
+        from .serializers import CrashEntryVueSerializer
+
+        inList, outList = [], []
+        inListCount, outListCount = 0, 0
+
+        signature = self.getSignature()
+        needTest = signature.matchRequiresTest()
+        entries = CrashEntry.objects.filter(
+            models.Q(bucket=None) | models.Q(bucket=self)
+        )
+        entries = entries.select_related(
+            "product", "platform", "os"
+        )  # these are used by getCrashInfo
+        if needTest:
+            entries = entries.select_related("testcase")
+
+        requiredOutputs = signature.getRequiredOutputSources()
+        entries = CrashEntry.deferRawFields(entries, requiredOutputs)
+
+        if not submitSave:
+            entries = entries.select_related("tool").order_by(
+                "-id"
+            )  # used by the preview list
+
+        entry_ids = entries.values_list("id", flat=True)
+
+        # from the python documentation (itertools)
+        def grouper(iterable, n, fillvalue=None):
+            """Collect data into fixed-length chunks or blocks"""
+            # grouper('ABCDEFG', 3, 'x') --> ABC DEF Gxx
+            args = [iter(iterable)] * n
+            return zip_longest(*args, fillvalue=fillvalue)
+
+        # If we are saving, we only care about the id of each entry
+        # Otherwise, we save the entire object. Limit to the first 100 entries to avoid
+        # OOM.
+        for entry_ids_chunk in grouper(entry_ids, 100):
+            entries_chunk = entries.filter(id__in=entry_ids_chunk)
+
+            for entry in entries_chunk:
+                match = signature.matches(
+                    entry.getCrashInfo(
+                        attachTestcase=needTest, requiredOutputSources=requiredOutputs
+                    )
+                )
+                if match and entry.bucket_id is None:
+                    if submitSave:
+                        inList.append(entry.pk)
+                    elif len(inList) < 100:
+                        inList.append(CrashEntryVueSerializer(entry).data)
+                    inListCount += 1
+                elif not match and entry.bucket_id is not None:
+                    if submitSave:
+                        outList.append(entry.pk)
+                    elif len(outList) < 100:
+                        outList.append(CrashEntryVueSerializer(entry).data)
+                    outListCount += 1
+
+        if submitSave:
+            while inList:
+                updList, inList = inList[:500], inList[500:]
+                for crash in CrashEntry.objects.filter(pk__in=updList).values(
+                    "bucket_id", "created", "tool_id"
+                ):
+                    if crash["bucket_id"] != self.id:
+                        if crash["bucket_id"] is not None:
+                            BucketHit.decrement_count(
+                                crash["bucket_id"], crash["tool_id"], crash["created"]
+                            )
+                        BucketHit.increment_count(
+                            self.id, crash["tool_id"], crash["created"]
+                        )
+                CrashEntry.objects.filter(pk__in=updList).update(bucket=self)
+            while outList:
+                updList, outList = outList[:500], outList[500:]
+                for crash in CrashEntry.objects.filter(pk__in=updList).values(
+                    "bucket_id", "created", "tool_id"
+                ):
+                    if crash["bucket_id"] is not None:
+                        BucketHit.decrement_count(
+                            crash["bucket_id"], crash["tool_id"], crash["created"]
+                        )
+                CrashEntry.objects.filter(pk__in=updList).update(
+                    bucket=None, triagedOnce=False
+                )
+
+        return inList, outList, inListCount, outListCount
 
     def optimizeSignature(self, unbucketed_entries):
         buckets = Bucket.objects.all()
@@ -138,19 +264,23 @@ class Bucket(models.Model):
         firstEntryPerBucketCache = {}
 
         for entry in entries:
-            entry.crashinfo = entry.getCrashInfo(attachTestcase=signature.matchRequiresTest(),
-                                                 requiredOutputSources=requiredOutputs)
+            entry.crashinfo = entry.getCrashInfo(
+                attachTestcase=signature.matchRequiresTest(),
+                requiredOutputSources=requiredOutputs,
+            )
 
-            # For optimization, disregard any issues that directly match since those could be
-            # incoming new issues and we don't want these to block the optimization.
+            # For optimization, disregard any issues that directly match since those
+            # could be incoming new issues and we don't want these to block the
+            # optimization.
             if signature.matches(entry.crashinfo):
                 continue
 
             optimizedSignature = signature.fit(entry.crashinfo)
             if optimizedSignature:
-                # We now try to determine how this signature will behave in other buckets
-                # If the signature matches lots of other buckets as well, it is likely too
-                # broad and we should not consider it (or later rate it worse than others).
+                # We now try to determine how this signature will behave in other
+                # buckets. If the signature matches lots of other buckets as well, it is
+                # likely too broad and we should not consider it (or later rate it worse
+                # than others).
                 matchesInOtherBuckets = False
                 nonMatchesInOtherBuckets = 0  # noqa
                 otherMatchingBucketIds = []  # noqa
@@ -158,12 +288,19 @@ class Bucket(models.Model):
                     if otherBucket.pk == self.pk:
                         continue
 
-                    if self.bug and otherBucket.bug and self.bug.pk == otherBucket.bug.pk:
-                        # Allow matches in other buckets if they are both linked to the same bug
+                    if (
+                        self.bug
+                        and otherBucket.bug
+                        and self.bug.pk == otherBucket.bug.pk
+                    ):
+                        # Allow matches in other buckets if they are both linked to the
+                        # same bug
                         continue
 
                     if otherBucket.pk not in firstEntryPerBucketCache:
-                        c = CrashEntry.objects.filter(bucket=otherBucket).select_related("product", "platform", "os")
+                        c = CrashEntry.objects.filter(
+                            bucket=otherBucket
+                        ).select_related("product", "platform", "os")
                         c = CrashEntry.deferRawFields(c, requiredOutputs)
                         c = c.first()
                         firstEntryPerBucketCache[otherBucket.pk] = c
@@ -171,7 +308,7 @@ class Bucket(models.Model):
                             # Omit testcase for performance reasons for now
                             firstEntryPerBucketCache[otherBucket.pk] = c.getCrashInfo(
                                 attachTestcase=False,
-                                requiredOutputSources=requiredOutputs
+                                requiredOutputSources=requiredOutputs,
                             )
 
                     firstEntryCrashInfo = firstEntryPerBucketCache[otherBucket.pk]
@@ -182,17 +319,19 @@ class Bucket(models.Model):
                             break
 
                 if matchesInOtherBuckets:
-                    # Reset, we don't actually have an optimized signature if it's matching
-                    # some other bucket as well.
+                    # Reset, we don't actually have an optimized signature if it's
+                    # matching some other bucket as well.
                     optimizedSignature = None
                 else:
                     for otherEntry in entries:
-                        otherEntry.crashinfo = otherEntry.getCrashInfo(attachTestcase=False,
-                                                                       requiredOutputSources=requiredOutputs)
+                        otherEntry.crashinfo = otherEntry.getCrashInfo(
+                            attachTestcase=False, requiredOutputSources=requiredOutputs
+                        )
                         if optimizedSignature.matches(otherEntry.crashinfo):
                             matchingEntries.append(otherEntry)
 
-                    # Fallback for when the optimization algorithm failed for some reason
+                    # Fallback for when the optimization algorithm failed for some
+                    # reason
                     if not matchingEntries:
                         optimizedSignature = None
 
@@ -201,15 +340,51 @@ class Bucket(models.Model):
         return (optimizedSignature, matchingEntries)
 
 
+def buckethit_default_range_begin():
+    return timezone.now().replace(microsecond=0, second=0, minute=0)
+
+
+class BucketHit(models.Model):
+    bucket = models.ForeignKey(Bucket, on_delete=models.deletion.CASCADE)
+    tool = models.ForeignKey(Tool, on_delete=models.deletion.CASCADE)
+    begin = models.DateTimeField(default=buckethit_default_range_begin)
+    count = models.IntegerField(default=0)
+
+    @classmethod
+    def decrement_count(cls, bucket_id, tool_id, begin):
+        begin = begin.replace(microsecond=0, second=0, minute=0)
+        counter = cls.objects.filter(
+            bucket_id=bucket_id,
+            begin=begin,
+            tool_id=tool_id,
+        ).first()
+        if counter is not None and counter.count > 0:
+            counter.count -= 1
+            counter.save()
+
+    @classmethod
+    def increment_count(cls, bucket_id, tool_id, begin):
+        begin = begin.replace(microsecond=0, second=0, minute=0)
+        counter, _ = cls.objects.get_or_create(
+            bucket_id=bucket_id, begin=begin, tool_id=tool_id
+        )
+        counter.count += 1
+        counter.save()
+
+
 class CrashEntry(models.Model):
     created = models.DateTimeField(default=timezone.now)
     tool = models.ForeignKey(Tool, on_delete=models.deletion.CASCADE)
     platform = models.ForeignKey(Platform, on_delete=models.deletion.CASCADE)
     product = models.ForeignKey(Product, on_delete=models.deletion.CASCADE)
     os = models.ForeignKey(OS, on_delete=models.deletion.CASCADE)
-    testcase = models.ForeignKey(TestCase, blank=True, null=True, on_delete=models.deletion.CASCADE)
+    testcase = models.ForeignKey(
+        TestCase, blank=True, null=True, on_delete=models.deletion.CASCADE
+    )
     client = models.ForeignKey(Client, on_delete=models.deletion.CASCADE)
-    bucket = models.ForeignKey(Bucket, blank=True, null=True, on_delete=models.deletion.CASCADE)
+    bucket = models.ForeignKey(
+        Bucket, blank=True, null=True, on_delete=models.deletion.CASCADE
+    )
     rawStdout = models.TextField(blank=True)
     rawStderr = models.TextField(blank=True)
     rawCrashData = models.TextField(blank=True)
@@ -232,19 +407,26 @@ class CrashEntry(models.Model):
         # automatically here. You need to explicitly call the
         # deserializeFields method if you need this data.
 
-        super(CrashEntry, self).__init__(*args, **kwargs)
+        self._original_bucket = None
+        super().__init__(*args, **kwargs)
+
+    @classmethod
+    def from_db(cls, db, field_names, values):
+        instance = super().from_db(db, field_names, values)
+        instance._original_bucket = instance.bucket_id
+        return instance
 
     def save(self, *args, **kwargs):
-        if self.pk is None and not getattr(settings, 'DB_ISUTF8MB4', False):
+        if self.pk is None and not getattr(settings, "DB_ISUTF8MB4", False):
             # Replace 4-byte UTF-8 characters with U+FFFD if our database
             # doesn't support them. By default, MySQL utf-8 does not support these.
-            utf8_4byte_re = re.compile(u'[^\u0000-\uD7FF\uE000-\uFFFF]', re.UNICODE)
+            utf8_4byte_re = re.compile("[^\u0000-\uD7FF\uE000-\uFFFF]", re.UNICODE)
 
             def sanitize_utf8(s):
-                if not isinstance(s, six.text_type):
-                    s = six.text_type(s, 'utf-8')
+                if not isinstance(s, str):
+                    s = str(s, "utf-8")
 
-                return utf8_4byte_re.sub(u"\uFFFD", s)
+                return utf8_4byte_re.sub("\uFFFD", s)
 
             self.rawStdout = sanitize_utf8(self.rawStdout)
             self.rawStderr = sanitize_utf8(self.rawStderr)
@@ -274,13 +456,13 @@ class CrashEntry(models.Model):
 
             # We need to possibly convert the numeric crash address from unsigned
             # to signed in order to store it in the database.
-            if (self.crashAddressNumeric > (2 ** 63 - 1)):
-                self.crashAddressNumeric -= 2 ** 64
+            if self.crashAddressNumeric > (2**63 - 1):
+                self.crashAddressNumeric -= 2**64
 
         if len(self.shortSignature) > 255:
             self.shortSignature = self.shortSignature[0:255]
 
-        super(CrashEntry, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
 
     def deserializeFields(self):
         if self.args:
@@ -288,16 +470,22 @@ class CrashEntry(models.Model):
 
         if self.env:
             envDict = json.loads(self.env)
-            self.envList = ["%s=%s" % (s, envDict[s]) for s in envDict.keys()]
+            self.envList = [f"{s}={envDict[s]}" for s in envDict.keys()]
 
         if self.metadata:
             metadataDict = json.loads(self.metadata)
-            self.metadataList = ["%s=%s" % (s, metadataDict[s]) for s in metadataDict.keys()]
+            self.metadataList = [f"{s}={metadataDict[s]}" for s in metadataDict.keys()]
 
-    def getCrashInfo(self, attachTestcase=False, requiredOutputSources=("stdout", "stderr", "crashdata")):
+    def getCrashInfo(
+        self,
+        attachTestcase=False,
+        requiredOutputSources=("stdout", "stderr", "crashdata"),
+    ):
         # TODO: This should be cached at some level
         # TODO: Need to include environment and program arguments here
-        configuration = ProgramConfiguration(self.product.name, self.platform.name, self.os.name, self.product.version)
+        configuration = ProgramConfiguration(
+            self.product.name, self.platform.name, self.os.name, self.product.version
+        )
 
         cachedCrashInfo = None
         if self.cachedCrashInfo:
@@ -314,8 +502,13 @@ class CrashEntry(models.Model):
         if cachedCrashInfo is None or "crashdata" in requiredOutputSources:
             rawCrashData = self.rawCrashData
 
-        crashInfo = CrashInfo.fromRawCrashData(rawStdout, rawStderr, configuration, rawCrashData,
-                                               cacheObject=cachedCrashInfo)
+        crashInfo = CrashInfo.fromRawCrashData(
+            rawStdout,
+            rawStderr,
+            configuration,
+            rawCrashData,
+            cacheObject=cachedCrashInfo,
+        )
 
         if attachTestcase and self.testcase is not None and not self.testcase.isBinary:
             self.testcase.loadTest()
@@ -333,7 +526,7 @@ class CrashEntry(models.Model):
         self.cachedCrashInfo = None
         crashInfo = self.getCrashInfo()
         if crashInfo.crashAddress is not None:
-            self.crashAddress = '0x%x' % crashInfo.crashAddress
+            self.crashAddress = f"0x{crashInfo.crashAddress:x}"
         self.shortSignature = crashInfo.createShortSignature()
 
         # If the entry has a bucket, check if it still fits into
@@ -356,21 +549,25 @@ class CrashEntry(models.Model):
         # This method calls defer() on the given query set for every raw field
         # that is not required as specified in requiredOutputSources.
         if "stdout" not in requiredOutputSources:
-            queryset = queryset.defer('rawStdout')
+            queryset = queryset.defer("rawStdout")
         if "stderr" not in requiredOutputSources:
-            queryset = queryset.defer('rawStderr')
+            queryset = queryset.defer("rawStderr")
         if "crashdata" not in requiredOutputSources:
-            queryset = queryset.defer('rawCrashData')
+            queryset = queryset.defer("rawCrashData")
         return queryset
 
 
 # These post_delete handlers ensure that the corresponding testcase
-# is also deleted when the CrashEntry is gone. It also explicitely
+# is also deleted when the CrashEntry is gone. It also explicitly
 # deletes the file on the filesystem which would otherwise remain.
 @receiver(post_delete, sender=CrashEntry)
 def CrashEntry_delete(sender, instance, **kwargs):
     if instance.testcase:
         instance.testcase.delete(False)
+    if instance.bucket_id is not None:
+        BucketHit.decrement_count(
+            instance.bucket_id, instance.tool_id, instance.created
+        )
 
 
 @receiver(post_delete, sender=TestCase)
@@ -379,15 +576,47 @@ def TestCase_delete(sender, instance, **kwargs):
         instance.test.delete(False)
 
 
-# post_save handler for celery integration
-if getattr(settings, 'USE_CELERY', None):
-    @receiver(post_save, sender=CrashEntry)
-    def CrashEntry_save(sender, instance, **kwargs):
-        if kwargs.get('created', False) and not instance.triagedOnce:
+@receiver(post_save, sender=CrashEntry)
+def CrashEntry_save(sender, instance, created, **kwargs):
+    if getattr(settings, "USE_CELERY", None):
+        if created and not instance.triagedOnce:
             triage_new_crash.delay(instance.pk)
 
+    if instance.bucket_id != instance._original_bucket:
+        if instance._original_bucket is not None:
+            # remove BucketHit for old bucket/tool
+            BucketHit.decrement_count(
+                instance._original_bucket, instance.tool_id, instance.created
+            )
+
+        if instance.bucket is not None:
+            # add BucketHit for new bucket
+            BucketHit.increment_count(
+                instance.bucket_id, instance.tool_id, instance.created
+            )
+
+        if instance.bucket is not None:
+            notify.send(
+                instance.bucket,
+                recipient=instance.bucket.watchers,
+                actor=instance.bucket,
+                verb="bucket_hit",
+                target=instance,
+                level="info",
+                description=(
+                    f"The bucket {instance.bucket_id} received a new crash entry "
+                    f"{instance.pk}"
+                ),
+            )
+
+
+class BugzillaTemplateMode(Enum):
+    Bug = "bug"
+    Comment = "comment"
+
 
 class BugzillaTemplate(models.Model):
+    mode = EnumField(BugzillaTemplateMode, max_length=30)
     name = models.TextField()
     product = models.TextField()
     component = models.TextField()
@@ -410,6 +639,11 @@ class BugzillaTemplate(models.Model):
     security_group = models.TextField(blank=True)
     comment = models.TextField(blank=True)
     testcase_filename = models.TextField(blank=True)
+    blocks = models.TextField(blank=True)
+    dependson = models.TextField(blank=True)
+
+    def __str__(self):
+        return self.name
 
 
 class User(models.Model):
@@ -428,12 +662,16 @@ class User(models.Model):
     defaultProviderId = models.IntegerField(default=1)
     defaultToolsFilter = models.ManyToManyField(Tool)
     restricted = models.BooleanField(blank=False, default=False)
-    bucketsWatching = models.ManyToManyField(Bucket, through='BucketWatch')
+    bucketsWatching = models.ManyToManyField(Bucket, through="BucketWatch")
+
+    # Notifications
+    inaccessible_bug = models.BooleanField(blank=False, default=False)
+    bucket_hit = models.BooleanField(blank=False, default=False)
 
     @staticmethod
     def get_or_create_restricted(request_user):
         (user, created) = User.objects.get_or_create(user=request_user)
-        if created and getattr(settings, 'USERS_RESTRICTED_BY_DEFAULT', False):
+        if created and getattr(settings, "USERS_RESTRICTED_BY_DEFAULT", False):
             user.restricted = True
             user.save()
         return (user, created)
@@ -442,15 +680,17 @@ class User(models.Model):
 @receiver(post_save, sender=DjangoUser)
 def add_default_perms(sender, instance, created, **kwargs):
     if created:
-        log = logging.getLogger('crashmanager')
-        for perm in getattr(settings, 'DEFAULT_PERMISSIONS', []):
-            model, perm = perm.split(':', 1)
-            module, model = model.rsplit('.', 1)
-            module = __import__(module, globals(), locals(), [model], 0)  # from module import model
+        log = logging.getLogger("crashmanager")
+        for perm in getattr(settings, "DEFAULT_PERMISSIONS", []):
+            model, perm = perm.split(":", 1)
+            module, model = model.rsplit(".", 1)
+            module = __import__(
+                module, globals(), locals(), [model], 0
+            )  # from module import model
             content_type = ContentType.objects.get_for_model(getattr(module, model))
             perm = Permission.objects.get(content_type=content_type, codename=perm)
             instance.user_permissions.add(perm)
-            log.info('user %s added permission %s:%s', instance.username, model, perm)
+            log.info("user %s added permission %s:%s", instance.username, model, perm)
 
 
 class BucketWatch(models.Model):
