@@ -1,9 +1,9 @@
 function getCSRFToken() {
-  let needle = "csrftoken="
+  const needle = "csrftoken="
   if (document.cookie && document.cookie !== '') {
-    let cookies = document.cookie.split(';');
+    const cookies = document.cookie.split(';');
     for (let i = 0; i < cookies.length; i++) {
-      let cookie = jQuery.trim(cookies[i]);
+      const cookie = cookies[i].trim();
       if (cookie.startsWith(needle)) {
         return decodeURIComponent(cookie.substring(needle.length));
       }
