@@ -1,5 +1,4 @@
-# coding: utf-8
-'''Common utilities for tests
+"""Common utilities for tests
 
 @author:     Jesse Schwartzentruber (:truber)
 
@@ -8,16 +7,14 @@
 This Source Code Form is subject to the terms of the Mozilla Public
 License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
-'''
+"""
 from django.test import SimpleTestCase as DjangoTestCase
 
 
 def assert_contains(response, text):
-    """Assert that the response was successful, and contains the given text.
-    """
+    """Assert that the response was successful, and contains the given text."""
 
     class _(DjangoTestCase):
-
         def runTest(self):
             pass
 
@@ -25,11 +22,9 @@ def assert_contains(response, text):
 
 
 def assert_not_contains(response, text):
-    """Assert that the response was successful, and does not contain the given text.
-    """
+    """Assert that the response was successful, and does not contain the given text."""
 
     class _(DjangoTestCase):
-
         def runTest(self):
             pass
 
