@@ -1,5 +1,4 @@
-# encoding: utf-8
-'''
+"""
 Prices -- Various methods for accessing price history on EC2
 
 @author:     Christian Holler (:decoder)
@@ -11,12 +10,13 @@ License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 @contact:    choller@mozilla.com
-'''
+"""
 
 
 def get_prices(regions, cloud_provider, instance_types=None, use_multiprocess=False):
     if use_multiprocess:
         from multiprocessing import Pool, cpu_count
+
         pool = Pool(cpu_count())
 
     try:
@@ -24,7 +24,9 @@ def get_prices(regions, cloud_provider, instance_types=None, use_multiprocess=Fa
         for region in regions:
             args = [region, instance_types]
             if use_multiprocess:
-                results.append(pool.apply_async(cloud_provider.get_prices_per_region, args))
+                results.append(
+                    pool.apply_async(cloud_provider.get_prices_per_region, args)
+                )
             else:
                 results.append(cloud_provider.get_prices_per_region(*args))
 
