@@ -1,5 +1,4 @@
-# coding: utf-8
-'''
+"""
 Common utilities for tests
 
 @author:     Jesse Schwartzentruber (:truber)
@@ -9,19 +8,24 @@ Common utilities for tests
 This Source Code Form is subject to the terms of the Mozilla Public
 License, v. 2.0. If a copy of the MPL was not distributed with this
 file, You can obtain one at http://mozilla.org/MPL/2.0/.
-'''
-from django.contrib.contenttypes.models import ContentType
-from django.contrib.auth.models import User, Permission
+"""
 import pytest
+from django.contrib.auth.models import Permission, User
+from django.contrib.contenttypes.models import ContentType
+
 from crashmanager.models import User as cmUser
 
 
-def _create_user(username, email="test@mozilla.com", password="test", has_permission=True):
+def _create_user(
+    username, email="test@mozilla.com", password="test", has_permission=True
+):
     user = User.objects.create_user(username, email, password)
     user.user_permissions.clear()
     if has_permission:
         content_type = ContentType.objects.get_for_model(cmUser)
-        perm = Permission.objects.get(content_type=content_type, codename='view_taskmanager')
+        perm = Permission.objects.get(
+            content_type=content_type, codename="view_taskmanager"
+        )
         user.user_permissions.add(perm)
     (user, _) = cmUser.get_or_create_restricted(user)
     user.save()
